@@ -14,7 +14,6 @@
 var STRUCT_LIST = set('struct', 'list');
 
 var addedLibraryItems = {};
-var asmLibraryFunctions = [];
 
 var allExternPrimitives = ['Math_floor', 'Math_abs', 'Math_sqrt', 'Math_pow',
   'Math_cos', 'Math_sin', 'Math_tan', 'Math_acos', 'Math_asin', 'Math_atan',
@@ -23,9 +22,6 @@ var allExternPrimitives = ['Math_floor', 'Math_abs', 'Math_sqrt', 'Math_pow',
   'Int8Array', 'Uint8Array', 'Int16Array', 'Uint16Array', 'Int32Array',
   'Uint32Array', 'Float32Array', 'Float64Array'];
 
-// Specifies the set of referenced built-in primitives such as Math.max etc.
-var usedExternPrimitives = {};
-
 var SETJMP_LABEL = -1;
 
 var INDENTATION = ' ';
@@ -94,22 +90,18 @@ function JSify(data, functionsOnly) {
     // line endings \r\n. This is undesirable, since line endings are managed in the form \n in the output for binary file writes, so
     // make sure the endings are uniform.
     snippet = snippet.toString().replace(/\r\n/gm,"\n");
-    assert(snippet.indexOf('XXX missing C define') == -1,
-           'Trying to include a library function with missing C defines: ' + finalName + ' | ' + snippet);
 
     // name the function; overwrite if it's already named
     snippet = snippet.replace(/function(?:\s+([^(]+))?\s*\(/, 'function ' + finalName + '(');
-    // Apply special js library debug modes
-    if (!LibraryManager.library[ident + '__asm']) {
-      // apply LIBRARY_DEBUG if relevant
-      if (LIBRARY_DEBUG) {
-        snippet = modifyFunction(snippet, function(name, args, body) {
-          return 'function ' + name + '(' + args + ') {\n' +
-                 'var ret = (function() { if (runtimeDebug) err("[library call:' + finalName + ': " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");\n' +
-                  body +
-                  '}).apply(this, arguments); if (runtimeDebug && typeof ret !== "undefined") err("  [     return:" + prettyPrint(ret)); return ret; \n}\n';
-        });
-      }
+
+    // apply LIBRARY_DEBUG if relevant
+    if (LIBRARY_DEBUG) {
+      snippet = modifyFunction(snippet, function(name, args, body) {
+        return 'function ' + name + '(' + args + ') {\n' +
+               'var ret = (function() { if (runtimeDebug) err("[library call:' + finalName + ': " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");\n' +
+                body +
+                '}).apply(this, arguments); if (runtimeDebug && typeof ret !== "undefined") err("  [     return:" + prettyPrint(ret)); return ret; \n}\n';
+      });
     }
     return snippet;
   }
@@ -158,7 +150,6 @@ function JSify(data, functionsOnly) {
       var noExport = false;
 
       if (allExternPrimitives.indexOf(ident) != -1) {
-        usedExternPrimitives[ident] = 1;
         return;
       } else if (!LibraryManager.library.hasOwnProperty(ident) && !LibraryManager.library.hasOwnProperty(ident + '__inline')) {
         if (!(finalName in IMPLEMENTED_FUNCTIONS) && !LINKABLE) {
@@ -295,7 +286,7 @@ function JSify(data, functionsOnly) {
       function addDependency(dep) {
         return addFromLibrary(dep, identDependents + ', referenced by ' + dependent);
       }
-      var depsText = (deps ? '\n' + deps.map(addDependency).filter(function(x) { return x != '' }).join('\n') : '');
+      var depsText = (deps ? deps.map(addDependency).filter(function(x) { return x != '' }).join('\n') + '\n' : '');
       var contentText;
       if (isFunction) {
         // Emit the body of a JS library function.
@@ -330,29 +321,11 @@ function JSify(data, functionsOnly) {
         contentText = 'var ' + finalName + '=' + snippet + ';';
       }
       var sig = LibraryManager.library[ident + '__sig'];
-      if (isFunction && sig && LibraryManager.library[ident + '__asm']) {
-        // asm library function, add it as generated code alongside the generated code
-        Functions.implementedFunctions[finalName] = sig;
-        asmLibraryFunctions.push(contentText);
-        contentText = ' ';
-        Functions.libraryFunctions[finalName] = 2;
-        noExport = true; // if it needs to be exported, that will happen in emscripten.py
-      }
       // asm module exports are done in emscripten.py, after the asm module is ready. Here
       // we also export library methods as necessary.
       if ((EXPORT_ALL || (finalName in EXPORTED_FUNCTIONS)) && !noExport) {
         contentText += '\nModule["' + finalName + '"] = ' + finalName + ';';
       }
-      if (!LibraryManager.library[ident + '__asm']) {
-        // If we are not an asm library func, and we have a dep that is, then we need to call
-        // into the asm module to reach that dep. so it must be exported from the asm module.
-        // We set EXPORTED_FUNCTIONS here to tell emscripten.py to do that.
-        deps.forEach(function(dep) {
-          if (LibraryManager.library[dep + '__asm']) {
-            EXPORTED_FUNCTIONS[mangleCSymbolName(dep)] = 0;
-          }
-        });
-      }
 
       var commentText = '';
       if (LibraryManager.library[ident + '__docs']) {
@@ -396,41 +369,7 @@ function JSify(data, functionsOnly) {
     if (!mainPass) {
       var generated = itemsDict.function.concat(itemsDict.type).concat(itemsDict.GlobalVariableStub).concat(itemsDict.GlobalVariable);
       print(generated.map(function(item) { return item.JS; }).join('\n'));
-
-      if (memoryInitialization.length > 0) {
-        // apply postsets directly into the big memory initialization
-        itemsDict.GlobalVariablePostSet = itemsDict.GlobalVariablePostSet.filter(function(item) {
-          var m;
-          if (m = /^HEAP([\dFU]+)\[([()>\d]+)\] *= *([()|\d{}\w_' ]+);?$/.exec(item.JS)) {
-            var type = getTypeFromHeap(m[1]);
-            var bytes = Runtime.getNativeTypeSize(type);
-            var target = eval(m[2]) << log2(bytes);
-            var value = m[3];
-            try {
-              value = eval(value);
-            } catch(e) {
-              // possibly function table {{{ FT_* }}} etc.
-              if (value.indexOf('{{ ') < 0) return true;
-            }
-            writeInt8s(memoryInitialization, target - Runtime.GLOBAL_BASE, value, type);
-            return false;
-          }
-          return true;
-        });
-        // write out the singleton big memory initialization value
-        if (USE_PTHREADS) {
-          print('if (!ENVIRONMENT_IS_PTHREAD) {') // Pthreads should not initialize memory again, since it's shared with the main thread.
-        }
-        print('/* memory initializer */ ' + makePointer(memoryInitialization, null, 'ALLOC_NONE', 'i8', 'GLOBAL_BASE', true));
-        if (USE_PTHREADS) {
-          print('}')
-        }
-      } else {
-        print('/* no memory initializer */'); // test purposes
-      }
-
       print('// {{PRE_LIBRARY}}\n'); // safe to put stuff here that statically allocates
-
       return;
     }
 
@@ -475,21 +414,6 @@ function JSify(data, functionsOnly) {
       print(preprocess(read('base64Utils.js')));
     }
 
-    var usedExternPrimitiveNames = Object.keys(usedExternPrimitives);
-    if (usedExternPrimitiveNames.length > 0) {
-      print('// ASM_LIBRARY EXTERN PRIMITIVES: ' + usedExternPrimitiveNames.join(',') + '\n');
-    }
-
-    if (asmLibraryFunctions.length > 0) {
-      print('// ASM_LIBRARY FUNCTIONS');
-      function fix(f) { // fix indenting to not confuse js optimizer
-        f = f.substr(f.indexOf('f')); // remove initial spaces before 'function'
-        f = f.substr(0, f.lastIndexOf('\n')+1); // remove spaces and last }  XXX assumes function has multiple lines
-        return f + '}'; // add unindented } to match function
-      }
-      print(asmLibraryFunctions.map(fix).join('\n'));
-    }
-
     if (abortExecution) throw Error('Aborting compilation due to previous errors');
 
     // This is the main 'post' pass. Print out the generated code that we have here, together with the
