@@ -13,7 +13,12 @@
 function processMacros(text) {
   return text.replace(/{{{([^}]|}(?!}))+}}}/g, function(str) {
     str = str.substr(3, str.length-6);
-    var ret = eval(str);
+    try {
+      var ret = eval(str);
+    } catch (ex) {
+      ex.stack = 'In the following macro:\n\n' + str + '\n\n' + ex.stack;
+      throw ex;
+    }
     return ret !== null ? ret.toString() : '';
   });
 }
@@ -372,7 +377,6 @@ function makeGlobalUse(ident) {
       return ident;
     }
     var ret = (Runtime.GLOBAL_BASE + index).toString();
-    if (SIDE_MODULE) ret = '(H_BASE+' + ret + ')';
     return ret;
   }
   return ident;
@@ -624,7 +628,7 @@ function parseNumerical(value, type) {
       if (value[0] === '-' && ret === 0) { return '-.0'; } // fix negative 0, toString makes it 0
     }
     if (type === 'double' || type === 'float') {
-      if (!RUNNING_JS_OPTS) ret = asmEnsureFloat(ret, type);
+      ret = asmEnsureFloat(ret, type);
     }
     return ret.toString();
   } else {
@@ -650,64 +654,6 @@ function parseLLVMString(str) {
   return ret;
 }
 
-// Generates the type signature for a structure, for each byte, the type that is there.
-// i32, 0, 0, 0 - for example, an int32 is here, then nothing to do for the 3 next bytes, naturally
-function generateStructTypes(type) {
-  if (isArray(type)) return type; // already in the form of [type, type,...]
-  if (Compiletime.isNumberType(type) || isPointerType(type)) {
-    if (type == 'i64') {
-      return ['i64', 0, 0, 0, 'i32', 0, 0, 0];
-    }
-    return [type].concat(zeros(Runtime.getNativeFieldSize(type)-1));
-  }
-
-  // Avoid multiple concats by finding the size first. This is much faster
-  var typeData = Types.types[type];
-  var size = typeData.flatSize;
-  var ret = new Array(size);
-  var index = 0;
-  function add(typeData) {
-    var array = typeData.name_[0] === '['; // arrays just have 2 elements in their fields, see calculateStructAlignment
-    var num = array ? parseInt(typeData.name_.substr(1)) : typeData.fields.length;
-    var start = index;
-    for (var i = 0; i < num; i++) {
-      var type = array ? typeData.fields[0] : typeData.fields[i];
-      if (!SAFE_HEAP && isPointerType(type)) type = '*'; // do not include unneeded type names without safe heap
-      if (Compiletime.isNumberType(type) || isPointerType(type)) {
-        if (type == 'i64') {
-          ret[index++] = 'i64';
-          ret[index++] = 0;
-          ret[index++] = 0;
-          ret[index++] = 0;
-          ret[index++] = 'i32';
-          ret[index++] = 0;
-          ret[index++] = 0;
-          ret[index++] = 0;
-          continue;
-        }
-        ret[index++] = type;
-      } else {
-        if (isStructType(type) && type[1] === '0') {
-          // this is [0 x something], which does nothing
-          // XXX this happens in java_nbody... assert(i === typeData.fields.length-1);
-          continue;
-        }
-        add(Types.types[type]);
-      }
-      var more = array ? (i+1)*typeData.flatSize/num : (
-        (i+1 < typeData.fields.length ? typeData.flatIndexes[i+1] : typeData.flatSize)
-      );
-      more -= index - start;
-      for (var j = 0; j < more; j++) {
-        ret[index++] = 0;
-      }
-    }
-  }
-  add(typeData);
-  assert(index == size);
-  return ret;
-}
-
 // Misc
 
 function indentify(text, indent) {
@@ -743,8 +689,7 @@ function ensureDot(value) {
   value = value.toString();
   // if already dotted, or Infinity or NaN, nothing to do here
   // if smaller than 1 and running js opts, we always need to force a coercion (0.001 will turn into 1e-3, which has no .)
-  if ((value.indexOf('.') >= 0 || /[IN]/.test(value)) && (!RUNNING_JS_OPTS || Math.abs(value) >= 1)) return value;
-  if (RUNNING_JS_OPTS) return '(+' + value + ')'; // JS optimizer will run, we must do +x, and it will be corrected later
+  if ((value.indexOf('.') >= 0 || /[IN]/.test(value))) return value;
   var e = value.indexOf('e');
   if (e < 0) return value + '.0';
   return value.substr(0, e) + '.0' + value.substr(e);
@@ -752,7 +697,7 @@ function ensureDot(value) {
 
 function asmEnsureFloat(value, type) { // ensures that a float type has either 5.5 (clearly a float) or +5 (float due to asm coercion)
   if (!isNumber(value)) return value;
-  if (PRECISE_F32 && type === 'float') {
+  if (type === 'float') {
     // normally ok to just emit Math_fround(0), but if the constant is large we may need a .0 (if it can't fit in an int)
     if (value == 0) return 'Math_fround(0)';
     value = ensureDot(value);
@@ -767,8 +712,8 @@ function asmEnsureFloat(value, type) { // ensures that a float type has either 5
 
 function asmInitializer(type) {
   if (type in Compiletime.FLOAT_TYPES) {
-    if (PRECISE_F32 && type === 'float') return 'Math_fround(0)';
-    return RUNNING_JS_OPTS ? '+0' : '.0';
+    if (type === 'float') return 'Math_fround(0)';
+    return '.0';
   } else {
     return '0';
   }
@@ -788,13 +733,16 @@ function asmCoercion(value, type, signedness) {
           value = '(' + value + ')|0';
         }
       }
-      if (PRECISE_F32 && type === 'float') {
+      if (type === 'float') {
         return 'Math_fround(' + value + ')';
       } else {
         return '(+(' + value + '))';
       }
     }
   } else {
+    if (signedness == 'u') {
+      return '((' + value + ')>>>0)';
+    }
     return '((' + value + ')|0)';
   }
 }
@@ -836,8 +784,7 @@ function makeGetValue(ptr, pos, type, noNeedFirst, unsigned, ignore, align, noSa
     return '{ ' + ret.join(', ') + ' }';
   }
 
-  // In double mode 1, in asmjs-unknown-emscripten we need this code path if we are not fully aligned.
-  if (DOUBLE_MODE == 1 && type == 'double' && (align < 8)) {
+  if (type == 'double' && (align < 8)) {
     return '(' + makeSetTempDouble(0, 'i32', makeGetValue(ptr, pos, 'i32', noNeedFirst, unsigned, ignore, align, noSafe)) + ',' +
                  makeSetTempDouble(1, 'i32', makeGetValue(ptr, getFastValue(pos, '+', Runtime.getNativeTypeSize('i32')), 'i32', noNeedFirst, unsigned, ignore, align, noSafe)) + ',' +
             makeGetTempDouble(0, 'double') + ')';
@@ -846,7 +793,6 @@ function makeGetValue(ptr, pos, type, noNeedFirst, unsigned, ignore, align, noSa
   if (align) {
     // Alignment is important here. May need to split this up
     var bytes = Runtime.getNativeTypeSize(type);
-    if (DOUBLE_MODE == 0 && type == 'double') bytes = 4; // we will really only read 4 bytes here
     if (bytes > align) {
       var ret = '(';
       if (isIntImplemented(type)) {
@@ -880,7 +826,7 @@ function makeGetValue(ptr, pos, type, noNeedFirst, unsigned, ignore, align, noSa
     if (printType !== 'null' && printType[0] !== '#') printType = '"' + safeQuote(printType) + '"';
     if (printType[0] === '#') printType = printType.substr(1);
     if (!ignore) {
-      return asmCoercion('SAFE_HEAP_LOAD' + ((type in Compiletime.FLOAT_TYPES) ? '_D' : '') + '(' + asmCoercion(offset, 'i32') + ', ' + Runtime.getNativeTypeSize(type) + ', ' + (!!unsigned+0) + ')', type);
+      return asmCoercion('SAFE_HEAP_LOAD' + ((type in Compiletime.FLOAT_TYPES) ? '_D' : '') + '(' + asmCoercion(offset, 'i32') + ', ' + Runtime.getNativeTypeSize(type) + ', ' + (!!unsigned+0) + ')', type, unsigned ? 'u' : undefined);
     }
   }
   var ret = makeGetSlabs(ptr, type, false, unsigned)[0] + '[' + getHeapOffset(offset, type) + ']';
@@ -904,7 +850,7 @@ function makeGetValueAsm(ptr, pos, type, unsigned) {
 //!             'null' means, in the context of SAFE_HEAP, that we should accept all types;
 //!             which means we should write to all slabs, ignore type differences if any on reads, etc.
 //! @param noNeedFirst Whether to ignore the offset in the pointer itself.
-function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign, forceAsm) {
+function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign) {
   sep = sep || ';';
   if (isStructType(type)) {
     var typeData = Types.types[type];
@@ -920,7 +866,7 @@ function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe,
     return ret.join('; ');
   }
 
-  if (DOUBLE_MODE == 1 && type == 'double' && (align < 8)) {
+  if (type == 'double' && (align < 8)) {
     return '(' + makeSetTempDouble(0, 'double', value) + ',' +
             makeSetValue(ptr, pos, makeGetTempDouble(0, 'i32'), 'i32', noNeedFirst, ignore, align, noSafe, ',') + ',' +
             makeSetValue(ptr, getFastValue(pos, '+', Runtime.getNativeTypeSize('i32')), makeGetTempDouble(1, 'i32'), 'i32', noNeedFirst, ignore, align, noSafe, ',') + ')';
@@ -935,7 +881,6 @@ function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe,
   if (align || needSplitting) {
     // Alignment is important here, or we need to split this up for other reasons.
     var bytes = Runtime.getNativeTypeSize(type);
-    if (DOUBLE_MODE == 0 && type == 'double') bytes = 4; // we will really only read 4 bytes here
     if (bytes > align || needSplitting) {
       var ret = '';
       if (isIntImplemented(type)) {
@@ -971,45 +916,8 @@ function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe,
   return makeGetSlabs(ptr, type, true).map(function(slab) { return slab + '[' + getHeapOffset(offset, type) + ']=' + value }).join(sep);
 }
 
-function makeSetValueAsm(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign) {
-  return makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign, true);
-}
-
 var UNROLL_LOOP_MAX = 8;
 
-function makeSetValues(ptr, pos, value, type, num, align) {
-  function unroll(type, num, jump, value$) {
-    jump = jump || 1;
-    value$ = value$ || value;
-    return range(num).map(function(i) {
-      return makeSetValue(ptr, getFastValue(pos, '+', i*jump), value$, type);
-    }).join('; ');
-  }
-  // If we don't know how to handle this at compile-time, or handling it is best done in a large amount of code, call memset
-  // TODO: optimize the case of numeric num but non-numeric value
-  if (!isNumber(num) || !isNumber(value) || (parseInt(num)/align >= UNROLL_LOOP_MAX)) {
-    return '_memset(' + asmCoercion(getFastValue(ptr, '+', pos), 'i32') + ', ' + asmCoercion(value, 'i32') + ', ' + asmCoercion(num, 'i32') + ')|0';
-  }
-  num = parseInt(num);
-  value = parseInt(value);
-  if (value < 0) value += 256; // make it unsigned
-  var values = {
-    1: value,
-    2: value | (value << 8),
-    4: value | (value << 8) | (value << 16) | (value << 24)
-  };
-  var ret = [];
-  [4, 2, 1].forEach(function(possibleAlign) {
-    if (num == 0) return;
-    if (align >= possibleAlign) {
-      ret.push(unroll('i' + (possibleAlign*8), Math.floor(num/possibleAlign), possibleAlign, values[possibleAlign]));
-      pos = getFastValue(pos, '+', Math.floor(num/possibleAlign)*possibleAlign);
-      num %= possibleAlign;
-    }
-  });
-  return ret.join('; ');
-}
-
 var TYPED_ARRAY_SET_MIN = Infinity; // .set() as memcpy seems to just slow us down
 
 function makeCopyValues(dest, src, num, type, modifier, align, sep) {
@@ -1051,17 +959,13 @@ function makeHEAPView(which, start, end) {
 // When dynamically linking, some things like dynCalls may not exist in one module and
 // be provided by a linked module, so they must be accessed indirectly using Module
 function exportedAsmFunc(func) {
-  if (!MAIN_MODULE && !SIDE_MODULE) {
+  if (!MAIN_MODULE) {
     return func;
   } else {
     return "Module['" + func + "']";
   }
 }
 
-function makeDynCall(sig) {
-  return exportedAsmFunc('dynCall_' + sig);
-}
-
 var TWO_TWENTY = Math.pow(2, 20);
 
 // Given two values and an operation, returns the result of that operation.
@@ -1134,7 +1038,7 @@ function getFastValue(a, op, b, type) {
       // if guaranteed small enough to not overflow into a double, do a normal multiply
       var bits = getBits(type) || 32; // default is 32-bit multiply for things like getelementptr indexes
       // Note that we can emit simple multiple in non-asm.js mode, but asm.js will not parse "16-bit" multiple, so must do imul there
-      if ((aNumber !== null && Math.abs(a) < TWO_TWENTY) || (bNumber !== null && Math.abs(b) < TWO_TWENTY) || (bits < 32 && !ASM_JS)) {
+      if ((aNumber !== null && Math.abs(a) < TWO_TWENTY) || (bNumber !== null && Math.abs(b) < TWO_TWENTY)) {
         return '(((' + a + ')*(' + b + '))&' + ((Math.pow(2, bits)-1)|0) + ')'; // keep a non-eliminatable coercion directly on this
       }
       return '(Math_imul(' + a + ',' + b + ')|0)';
@@ -1194,88 +1098,6 @@ var temp32f = new Float32Array(temp64f.buffer);
 var temp32 = new Uint32Array(temp64f.buffer);
 var temp16 = new Uint16Array(temp64f.buffer);
 var temp8 = new Uint8Array(temp64f.buffer);
-var memoryInitialization = [];
-
-function writeInt8s(slab, i, value, type) {
-  var currSize;
-  switch (type) {
-    case 'i1':
-    case 'i8': temp8[0] = value;       currSize = 1; break;
-    case 'i16': temp16[0] = value;     currSize = 2; break;
-    case 'float': temp32f[0] = value;  currSize = 4; break;
-    case 'double': temp64f[0] = value; currSize = 8; break;
-    case 'i64': // fall through, i64 is two i32 chunks
-    case 'i32': // fall through, i32 can be a pointer
-    default: {
-      if (type == 'i32' || type == 'i64' || type[type.length-1] == '*') {
-        if (!isNumber(value)) { // function table stuff, etc.
-          slab[i] = value;
-          slab[i+1] = slab[i+2] = slab[i+3] = 0;
-          return 4;
-        }
-        temp32[0] = value;
-        currSize = 4;
-      } else {
-        throw 'what? ' + types[i];
-      }
-    }
-  }
-  for (var j = 0; j < currSize; j++) {
-    slab[i+j] = temp8[j];
-  }
-  return currSize;
-}
-
-function makePointer(slab, pos, allocator, type, ptr, finalMemoryInitialization) {
-  assert(type, 'makePointer requires type info');
-  if (typeof slab == 'string' && (slab.substr(0, 4) === 'HEAP')) return pos;
-  var types = generateStructTypes(type);
-  if (typeof slab == 'object') {
-    for (var i = 0; i < slab.length; i++) {
-      var curr = slab[i];
-      if (isNumber(curr)) {
-        slab[i] = parseFloat(curr); // fix "5" to 5 etc.
-      } else if (curr == 'undef') {
-        slab[i] = 0;
-      }
-    }
-  }
-  // compress type info and data if possible
-  if (!finalMemoryInitialization) {
-    // XXX This heavily assumes the target endianness is the same as our current endianness! XXX
-    var i = 0;
-    while (i < slab.length) {
-      var currType = types[i];
-      if (!currType) { i++; continue }
-      i += writeInt8s(slab, i, slab[i], currType);
-    }
-    types = 'i8';
-  }
-  if (allocator == 'ALLOC_NONE') {
-    if (!finalMemoryInitialization) {
-      // writing out into memory, without a normal allocation. We put all of these into a single big chunk.
-      assert(typeof slab == 'object');
-      assert(slab.length % QUANTUM_SIZE == 0, slab.length); // must be aligned already
-      if (SIDE_MODULE && typeof ptr == 'string') {
-        ptr = parseInt(ptr.substring(ptr.indexOf('+'), ptr.length-1)); // parse into (H_BASE+X)
-      }
-      var offset = ptr - Runtime.GLOBAL_BASE;
-      for (var i = 0; i < slab.length; i++) {
-        memoryInitialization[offset + i] = slab[i];
-      }
-      return '';
-    }
-    // This is the final memory initialization
-    types = 'i8';
-  }
-
-  if (typeof types == 'object') {
-    while (types.length < slab.length) types.push(0);
-  }
-  types = JSON.stringify(types);
-  if (typeof slab == 'object') slab = '[' + slab.join(',') + ']';
-  return 'allocate(' + slab + ', ' + types + (allocator ? ', ' + allocator : '') + (allocator == 'ALLOC_NONE' ? ', ' + ptr : '') + ');';
-}
 
 function makeGetSlabs(ptr, type, allowMultiple, unsigned) {
   assert(type);
@@ -1453,18 +1275,16 @@ function ensureValidFFIType(type) {
 // FFI return values must arrive as doubles, and we can force them to floats afterwards
 function asmFFICoercion(value, type) {
   value = asmCoercion(value, ensureValidFFIType(type));
-  if (PRECISE_F32 && type === 'float') value = asmCoercion(value, 'float');
+  if (type === 'float') value = asmCoercion(value, 'float');
   return value;
 }
 
-function makeDynCall(sig) {
-  // asm.js function tables have one table in each linked asm.js module, so we
-  // can't just dynCall into them - ftCall exists for that purpose. In wasm,
-  // even linked modules share the table, so it's all fine.
-  if (EMULATED_FUNCTION_POINTERS && !WASM) {
-    return 'ftCall_' + sig;
+function makeDynCall(sig, funcPtr) {
+  assert(sig.indexOf('j') == -1);
+  if (USE_LEGACY_DYNCALLS) {
+    return `getDynCaller("${sig}", ${funcPtr})`;
   } else {
-    return 'dynCall_' + sig;
+    return `wasmTable.get(${funcPtr})`;
   }
 }
 
@@ -1487,19 +1307,6 @@ function makeEval(code) {
   return ret;
 }
 
-function makeStaticAlloc(size) {
-  size = alignMemory(size);
-  var ret = alignMemory(GLOBAL_BASE + STATIC_BUMP);
-  STATIC_BUMP = ret + size - GLOBAL_BASE;
-  return ret;
-}
-
-function makeStaticString(string) {
-  var len = lengthBytesUTF8(string) + 1;
-  var ptr = makeStaticAlloc(len);
-  return '(stringToUTF8("' + string + '", ' + ptr + ', ' + len + '), ' + ptr + ')';
-}
-
 var ATINITS = [];
 
 function addAtInit(code) {
@@ -1548,11 +1355,6 @@ function makeRetainedCompilerSettings() {
 // In wasm, the heap size must be a multiple of 64KB.
 // In asm.js, it must be a multiple of 16MB.
 var WASM_PAGE_SIZE = 65536;
-var ASMJS_PAGE_SIZE = 16777216;
-
-function getMemoryPageSize() {
-  return WASM ? WASM_PAGE_SIZE : ASMJS_PAGE_SIZE;
-}
 
 // Page size reported by some POSIX calls, mostly filesystem. This does not
 // depend on the memory page size which differs between wasm and asm.js, and
@@ -1731,3 +1533,18 @@ function addReadyPromiseAssertions(promise) {
     `;
   }).join('\n');
 }
+
+function makeMalloc(source, param) {
+  if ('_malloc' in IMPLEMENTED_FUNCTIONS) {
+    return '_malloc(' + param + ')';
+  }
+  // It should be impossible to call some functions without malloc being
+  // included, unless we have a deps_info.json bug. To let closure not error
+  // on `_malloc` not being present, they don't call malloc and instead abort
+  // with an error at runtime.
+  // TODO: A more comprehensive deps system could catch this at compile time.
+  if (!ASSERTIONS) {
+    return "abort();";
+  }
+  return `abort('malloc was not included, but is needed in ${source}. Adding "_malloc" to EXPORTED_FUNCTIONS should fix that. This may be a bug in the compiler, please file an issue.');`
+}
