@@ -1,7 +1,8 @@
-// Copyright 2019 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+//
+// @license
+// Copyright 2019 The Emscripten Authors
+// SPDX-License-Identifier: MIT
+//
 
 //
 // Settings in this file work exactly like those in settings.js but are not
@@ -61,18 +62,12 @@ var EMBIND = 0;
 // Whether the main() function reads the argc/argv parameters.
 var MAIN_READS_PARAMS = 1;
 
-// The computed location of the pointer to the sbrk position.
-var DYNAMICTOP_PTR = -1;
-
-// The computed initial value of the program break (the sbrk position), which
-// is called DYNAMIC_BASE as it is the start of dynamically-allocated memory.
-var DYNAMIC_BASE = -1;
-
-// -Werror was specified on the command line.
-var WARNINGS_ARE_ERRORS = 0;
-
-// Maximum seen global alignment; received from the backend.
-var MAX_GLOBAL_ALIGN = -1;
+// Computed during emscripten for the purpose of writing to emscripten_metadata
+// section.
+// TODO(sbc): Remove this.  If emscripten doesn't need it then neither should
+// any other loader.
+// See https://github.com/emscripten-core/emscripten/issues/12231
+var LEGACY_DYNAMIC_BASE = -1;
 
 // List of functions implemented in compiled code; received from the backend.
 var IMPLEMENTED_FUNCTIONS = [];
@@ -122,9 +117,6 @@ var WASM_TEXT_FILE = '';
 // name of the file containing wasm binary, if relevant
 var WASM_BINARY_FILE = '';
 
-// name of the file containing asm.js code, if relevant
-var ASMJS_CODE_FILE = '';
-
 // name of the file containing the pthread *.worker.js, if relevant
 var PTHREAD_WORKER_FILE = '';
 
@@ -137,12 +129,7 @@ var MEM_INIT_IN_WASM = 0;
 // This is set internally when needed (SINGLE_FILE)
 var SUPPORT_BASE64_EMBEDDING = 0;
 
-// the total static allocation, that is, how much to bump the start of memory
-// for static globals. received from the backend, and possibly increased due
-// to JS static allocations
-var STATIC_BUMP = -1;
-
-// the total initial wasm table size.
+// the total initial wasm table size, only used in RELOCATABLE mode
 var WASM_TABLE_SIZE = 0;
 
 // the possible environments the code may run in.
@@ -150,6 +137,7 @@ var ENVIRONMENT_MAY_BE_WEB = 1;
 var ENVIRONMENT_MAY_BE_WORKER = 1;
 var ENVIRONMENT_MAY_BE_NODE = 1;
 var ENVIRONMENT_MAY_BE_SHELL = 1;
+var ENVIRONMENT_MAY_BE_WEBVIEW = 1;
 
 // Whether to minify import and export names in the minify_wasm_js stage.
 var MINIFY_WASM_IMPORTS_AND_EXPORTS = 0;
@@ -157,11 +145,6 @@ var MINIFY_WASM_IMPORTS_AND_EXPORTS = 0;
 // Whether to minify imported module names.
 var MINIFY_WASM_IMPORTED_MODULES = 0;
 
-// passes information to emscripten.py about whether to minify
-// JS -> asm.js import names. Controlled by optimization level, enabled
-// at -O1 and higher, but disabled at -g2 and higher.
-var MINIFY_ASMJS_IMPORT_NAMES = 0;
-
 // Whether to minify functions exported from Asm.js/Wasm module.
 var MINIFY_ASMJS_EXPORT_NAMES = 1;
 
@@ -176,6 +159,10 @@ var WASM_FUNCTIONS_THAT_ARE_NOT_NAME_MANGLED = ['setTempRet0', 'getTempRet0', 's
 // Internal: value of -flto argument (either full or thin)
 var LTO = 0;
 
+// Whether we may be accessing the address 2GB or higher. If so then we need
+// to be using unsigned pointers in JS.
+var CAN_ADDRESS_2GB = 0;
+
 // Whether to emit DWARF in a separate wasm file on the side (this is not called
 // "split" because there is already a DWARF concept by that name).
 // When DWARF is on the side, the main file has no DWARF info, while the side
@@ -183,3 +170,28 @@ var LTO = 0;
 // sections.
 // This has no effect if DWARF is not being emitted.
 var SEPARATE_DWARF = 0;
+
+// New WebAssembly exception handling (experimental)
+var EXCEPTION_HANDLING = 0;
+
+// Used internally when running the JS compiler simply to generate list of all
+// JS symbols. This is used by LLD_REPORT_UNDEFINED to generate a list of all
+// JS library symbols.
+var ONLY_CALC_JS_SYMBOLS = 0;
+
+// Set to true if the program has a main function.  By default this is
+// enabled, but if `--no-entry` is passed, or if `_main` is not part of
+// EXPORTED_FUNCTIONS then this gets set to 0.
+var EXPECT_MAIN = 1;
+
+// Provide and export a .ready() Promise. This is currently used by default with
+// MODULARIZE, and returned from the factory function.
+var EXPORT_READY_PROMISE = 1;
+
+var USE_LEGACY_DYNCALLS = 0;
+
+// struct_info that is either generated or cached
+var STRUCT_INFO = '';
+
+// If true, building against Emscripten's asm.js/wasm heap memory profiler.
+var MEMORYPROFILER = 0;
