@@ -29,7 +29,14 @@ uintptr_t emscripten_stack_get_end(void);
 // Returns the current stack pointer.
 uintptr_t emscripten_stack_get_current(void);
 
-// Returns the number of free bytes left on the stack.
+// Setup internal state such that emscripten_stack_get_free can be used.
+// This needed until we can fix:
+// https://github.com/emscripten-core/emscripten/issues/11773
+void emscripten_stack_init(void);
+
+// Returns the number of free bytes left on the stack.  This is required to be
+// fast so that it can be called frequently and requires `emscripten_stack_init`
+// to be called before it will return accurate values.
 size_t emscripten_stack_get_free(void);
 
 #ifdef __cplusplus
